#ifndef	   COMPAT_STRINGSTREAMBUF_H_INCLUDED
#define	   COMPAT_STRINGSTREAMBUF_H_INCLUDED

#ifdef HAVE_CONFIG_H
# include  "config.h"
#endif

#include  "streambuf_compat_wrapper.h"
#include  <string>

class  compat_stringstreambuf : public std::streambuf
{
protected:
	std::string	buf;

private:
	 // not allowed.
	 compat_stringstreambuf( const compat_stringstreambuf & );

	 // not allowed.
	 compat_stringstreambuf &
		     operator= ( const compat_stringstreambuf & );

public:
		compat_stringstreambuf()
			: std::streambuf() , buf()
		{
		}

		compat_stringstreambuf( const std::string &  input )
			: std::streambuf() , buf( input )
		{
		}

	virtual	~compat_stringstreambuf()
		{
		}

protected:
	virtual	int  in_avail()
		{
			return( this -> buf.size() );
		}

	virtual	int  overflow( int  ch = EOF )
		{
			if ( ch == EOF )
			{
				return( EOF );
			}

			this -> buf.append( 1 , static_cast<char>(ch) );

			return( ch );
		}

	virtual	int  underflow()
		{
			if ( this -> buf.size() >= 1 )
			{
				return( static_cast<unsigned char>
					( this -> buf[0] ) );
			}
			else
			{
				return( EOF );
			}
		}

	virtual	int  uflow()
		{
			if ( this -> buf.size() >= 1 )
			{
				unsigned char	ch = this -> buf[0];

				this -> buf.erase( this -> buf.begin() );

				return( ch );
			}
			else
			{
				return( EOF );
			}
		}

	virtual	int  pbackfail( int  ch = EOF )
		{
			if ( ch != EOF )
			{
				this -> buf.insert( this -> buf.begin() ,
						    static_cast<char>(ch) );
			}

			return( ch );
		}

	virtual std::streamsize xsputn( const char *  s ,  std::streamsize  n )
		{
			this -> buf.append( s , n );

			return( n );
		}

	virtual std::streamsize xsgetn( char *  s ,  std::streamsize  n )
		{
			std::streamsize	total = this -> buf.copy( s , n );

			this -> buf.erase( 0 , total );

			return( total );
		}

public:
	virtual	const std::string &	str() const
		{
			return( this -> buf );
		}
};


#endif	/* COMPAT_STRINGSTREAMBUF_H_INCLUDED */
